/* License file snipped from here. Please see the "LICENSE" file, or the top of
 * the "swamp.h" header. Both contain the current license. */


#ifndef HEADER_DIST_PATTERN_H
#define HEADER_DIST_PATTERN_H

/*
 * Distribution patterns.
 *
 * These store the list of servers swamp will use and store a pattern of indexes
 * referring to those servers. The pattern controls the order (and frequency)
 * with which each server is 'swamp'ed (see 'server_iterator' to see how each
 * swamp context uses this pattern).
 *
 * To add a server to the list;
 *
 *    dist_pattern_push_address()
 *
 * To create the pattern that will be used on the servers:
 *
 *    dist_pattern_parse()
 *
 *    - If passed a non-NULL string, it parses the string as a comma and/or
 *      space separated list of indexes to the server list (eg.
 *      "1,2,3,4,5,2,4"). The indexes must be valid, ie. between 1 and the
 *      number of servers in the list. Once parsed into 'dist_pattern', the
 *      indexes are stored zero-based.
 *
 *    - If passed a NULL string, a canonical default pattern is generated to
 *      cycle across the servers in order once each. Ie. if there are 5 servers,
 *      a pattern of "1,2,3,4,5" is assumed.
 *
 */

/* Learn about "swamp_address" */
#include "net.h"

/* This opaque type represents the pattern and order of servers to swamp */
typedef struct st_dist_pattern dist_pattern;

/* Error return codes for dist_pattern_* functions. */
typedef enum {
	ERR_DIST_PAT_OKAY = 0,
	ERR_DIST_PAT_VALUE_OUT_OF_RANGE,
	ERR_DIST_PAT_INVALID_SYNTAX,
	ERR_DIST_PAT_ARRAY_FULL,
	ERR_DIST_PAT_INTERNAL_PROBLEM
} dist_pattern_error_t;

/**************************/
/* dist_pattern functions */
/**************************/

void dist_pattern_up(dist_pattern *p);
dist_pattern *dist_pattern_new(void);
void dist_pattern_free(dist_pattern *dist);
unsigned int dist_pattern_get_start_idx(dist_pattern *p);
unsigned int dist_pattern_period(dist_pattern *dist);
unsigned int dist_pattern_num(dist_pattern *dist);
const swamp_address *dist_pattern_get(const dist_pattern *dist,
				unsigned int idx);
ipv4_parse_error_t dist_pattern_push_address(dist_pattern *dist,
				const char *address);
dist_pattern_error_t dist_pattern_parse(dist_pattern *dist,
				const char *dist_str);
const char *dist_pattern_error_string(dist_pattern_error_t err);

#endif /* !defined(HEADER_DIST_PATTERN_H) */
