/* License file snipped from here. Please see the "LICENSE" file, or the top of
 * the "swamp.h" header. Both contain the current license. */


#include "fifo.h"

/* If we want to debug the total volume of traffic read in and written out
 * (across all file-descriptors), uncomment this define */
/* #define FIFO_FD_DEBUG */
#define FIFO_FD_DEBUG_STEP 1048576 /* 1Mb */

/* Define our opaque type */
struct st_swamp_fifo {
	unsigned char *data;
	unsigned int used;
	unsigned int size;
};

/* Create/destroy a "fifo" */
swamp_fifo *swamp_fifo_new(unsigned int size)
{
	swamp_fifo *f;
	if(!size)
		return NULL;
	f = malloc(sizeof(swamp_fifo));
	if(!f)
		return NULL;
	f->data = malloc(size);
	if(!f->data) {
		free(f);
		return NULL;
	}
	f->size = size;
	f->used = 0;
	return f;
}
void swamp_fifo_free(swamp_fifo *f)
{
	free(f->data);
	free(f);
}

/* Boolean tests */
int swamp_fifo_isempty(const swamp_fifo *f)
{
	return (f->used == 0);
}
int swamp_fifo_isfull(const swamp_fifo *f)
{
	return (f->used == f->size);
}
int swamp_fifo_notempty(const swamp_fifo *f)
{
	return (f->used > 0);
}
int swamp_fifo_notfull(const swamp_fifo *f)
{
	return (f->used < f->size);
}

/* Other query functions */
unsigned int swamp_fifo_used(const swamp_fifo *f)
{
	return f->used;
}
unsigned int swamp_fifo_unused(const swamp_fifo *f)
{
	return (f->size - f->used);
}

/* Interaction between "fifo"s and OpenSSL's "BIO"s. Return value is the amount
 * of data moved in each case. */
unsigned int swamp_fifo_from_BIO(swamp_fifo *f, BIO *bio)
{
	int tmp;
	unsigned int max = swamp_fifo_unused(f);
	if(!max)
		/* no room to read into */
		return 0;
	tmp = BIO_read(bio, f->data + f->used, max);
	if(tmp > 0) {
		f->used += tmp;
		return tmp;
	}
	return 0;
}

unsigned int swamp_fifo_to_BIO(swamp_fifo *f, BIO *bio)
{
	int tmp;
	unsigned int max = swamp_fifo_used(f);
	if(!max)
		/* nothing to write */
		return 0;
	tmp = BIO_write(bio, f->data, f->used);
	if(tmp > 0) {
		f->used -= tmp;
		/* Do we have to scroll? */
		if(f->used)
			/* yes */
			memmove(f->data, f->data + tmp, f->used);
		return tmp;
	}
	return 0;
}

#ifdef FIFO_FD_DEBUG
typedef struct st_traffic_debug {
	const char *name;
	unsigned long amt;
	time_t stamp;
	unsigned long target;
} traffic_debug;
static traffic_debug traffic_in  = {"incoming", 0, (time_t)0, 0};
static traffic_debug traffic_out = {"outgoing", 0, (time_t)0, 0};
static void traffic_update(traffic_debug *d, unsigned int delta)
{
	time_t t;
	/* Special case */
	if(!d->target) {
		d->stamp = time(NULL);
		d->amt = 0;
		d->target = FIFO_FD_DEBUG_STEP;
	}
	d->amt += delta;
	if(d->amt < d->target)
		return;
	t = time(NULL);
	printf("FIFO_FD_DEBUG: '%s' traffic reached %lu. Last %lu at %f bytes/sec\n",
		d->name, d->target, FIFO_FD_DEBUG_STEP,
		(t == d->stamp) ? 0.0 : (float)FIFO_FD_DEBUG_STEP / (t - d->stamp));
	d->stamp = t;
	d->target += FIFO_FD_DEBUG_STEP;
}
#define DEBUG_INCOMING(amt) traffic_update(&traffic_in, amt)
#define DEBUG_OUTGOING(amt) traffic_update(&traffic_out, amt)
#else
#define DEBUG_INCOMING(amt)
#define DEBUG_OUTGOING(amt)
#endif

/* Interaction between "fifo"s and file-descriptors. */
ssize_t swamp_fifo_from_fd(swamp_fifo *f, int fd)
{
	ssize_t ret;
	unsigned int max = swamp_fifo_unused(f);
	if(!max)
		return 0;
	ret = recv(fd, f->data + f->used, max, RECV_FLAGS);
	if(ret < 0) {
		switch(errno) {
		case EINTR:
		case EAGAIN:
			return 0;
		default:
			return -1;
		}
	}
	DEBUG_INCOMING(ret);
	f->used += ret;
	return ret;
}

ssize_t swamp_fifo_to_fd(swamp_fifo *f, int fd)
{
	ssize_t ret;
	unsigned int max = swamp_fifo_used(f);
	if(!max)
		return 0;
	ret = send(fd, f->data, max, SEND_FLAGS);
	if(ret < 0) {
		switch(errno) {
		case EINTR:
		case EAGAIN:
			return 0;
		default:
			return -1;
		}
	}
	if(ret > 0) {
		f->used -= ret;
		if(f->used)
			/* we have to scroll data, it wasn't all written */
			memmove(f->data, f->data + ret, f->used);
		DEBUG_OUTGOING(ret);
	}
	return ret;
}
