/* License file snipped from here. Please see the "LICENSE" file, or the top of
 * the "swamp.h" header. Both contain the current license. */


/* Learn what we're implementing */
#include "net.h"

/***************************/
/* Define our opaque types */
/***************************/

/* The "address" type */
struct st_swamp_address {
	/* The IPv4 address */
	unsigned char ipv4[4];
	/* The IP port */
	in_port_t port;
	/* The string that created this address */
	const char *original_string;
};

/* The "connection" type */
struct st_swamp_connection {
	/* The file-descriptor of the connection */
	int fd;
	/* FIFO buffers for incoming and outgoing data */
	swamp_fifo *fifo_in;
	swamp_fifo *fifo_out;
};

/* An internal-only selector "group" */
typedef struct st_selector_group {
	fd_set reads;
	fd_set sends;
	fd_set excepts;
	int max;
} selector_group;

/* The "selector" type */
struct st_swamp_selector {
	selector_group done;
	selector_group next;
};

/*************/
/* Constants */
/*************/

/* These flags are used from selector_group_get_conn() to return the status of a
 * connection after a select. */
#define SG_FLAG_READ	(unsigned int)0x0001
#define SG_FLAG_SEND	(unsigned int)0x0002
#define SG_FLAG_EXCEPT	(unsigned int)0x0004

/***************************/
/* swamp_address functions */
/***************************/

/* Create/destroy an "address" */
swamp_address *swamp_address_new(const char *address_string,
				ipv4_parse_error_t *error_val)
{
	ipv4_parse_error_t ev;
	swamp_address *sa = malloc(sizeof(swamp_address));
	if(!sa)
		ev = ERR_IPV4_PARSE_MEM_FAILURE;
	else
		ev = util_parse_ipv4_address(address_string,
				sa->ipv4, &sa->port);
	if(error_val)
		*error_val = ev;
	if(sa && (ev != ERR_IPV4_PARSE_OKAY)) {
		free(sa);
		return NULL;
	}
	sa->original_string = address_string;
	return sa;
}
void swamp_address_free(swamp_address *sa)
{
	free(sa);
}

/* Query functions */
const unsigned char *swamp_address_get_ip(const swamp_address *item)
{ return item->ipv4; }
in_port_t swamp_address_get_port(const swamp_address *item)
{ return item->port; }
const char *swamp_address_get_string(const swamp_address *item)
{ return item->original_string; }

/******************************/
/* swamp_connection functions */
/******************************/

swamp_connection *swamp_connection_new(const swamp_address *address,
				unsigned int buffer_size)
{
	int flags;
	struct sockaddr_in sa;
	swamp_connection *c = malloc(sizeof(swamp_connection));
	if(!c)
		goto fail;
	/* Avoid failures core-dumping in some combinations */
	c->fifo_out = NULL;
	c->fd = -1;
	if(!(c->fifo_in = swamp_fifo_new(buffer_size)) ||
			!(c->fifo_out = swamp_fifo_new(buffer_size)))
		goto fail;
	sa.sin_port = htons(swamp_address_get_port(address));
	memcpy(&sa.sin_addr, swamp_address_get_ip(address), 4);
	sa.sin_family = AF_INET;
	if((c->fd = socket(AF_INET, SOCK_STREAM, 0)) == -1)
		goto fail;
	if(((flags = fcntl(c->fd, F_GETFL, 0)) < 0) ||
			(fcntl(c->fd, F_SETFL, flags | O_NONBLOCK) < 0))
		goto fail;
	if((connect(c->fd, (struct sockaddr *)&sa, sizeof(sa)) == -1) &&
			(errno != EINPROGRESS))
		goto fail;
	return c;
fail:
	if(c) {
		if(c->fifo_in) swamp_fifo_free(c->fifo_in);
		if(c->fifo_out) swamp_fifo_free(c->fifo_out);
		if(c->fd != -1)
			close(c->fd);
		free(c);
	}
	return NULL;
}

void swamp_connection_free(swamp_connection *c)
{
	swamp_fifo_free(c->fifo_in);
	swamp_fifo_free(c->fifo_out);
	if(c->fd != -1)
		close(c->fd);
	free(c);
}

void swamp_connection_tofrom_BIO(swamp_connection *c,
				BIO *bio_fromconnection,
				BIO *bio_toconnection)
{
	swamp_fifo_from_BIO(c->fifo_out, bio_toconnection);
	swamp_fifo_to_BIO(c->fifo_in, bio_fromconnection);
}

/* Get (const) access to the incoming and outgoing "fifo"s */
const swamp_fifo *swamp_connection_get_read(swamp_connection *c)
{
	return c->fifo_in;
}
const swamp_fifo *swamp_connection_get_send(swamp_connection *c)
{
	return c->fifo_out;
}

/************************************************/
/* Internal-only functions for 'selector_group' */
/************************************************/

static void selector_group_clear(selector_group *sg) {
	FD_ZERO(&sg->reads);
	FD_ZERO(&sg->sends);
	FD_ZERO(&sg->excepts);
	sg->max = 0;
}
static void selector_group_add_conn(selector_group *sg,
				const swamp_connection *c) {
	/* Do we add for reading? */
	if(swamp_fifo_notfull(c->fifo_in))
		FD_SET(c->fd, &sg->reads);
	/* Do we add for writing? */
	if(swamp_fifo_notempty(c->fifo_out))
		FD_SET(c->fd, &sg->sends);
	FD_SET(c->fd, &sg->excepts);
	if(sg->max <= c->fd)
		sg->max = (c->fd + 1);
}
static unsigned int selector_group_get_conn(selector_group *sg,
				const swamp_connection *c) {
	unsigned int toret = 0;
	if(FD_ISSET(c->fd, &sg->reads))
		toret = SG_FLAG_READ;
	if(FD_ISSET(c->fd, &sg->sends))
		toret |= SG_FLAG_SEND;
	if(FD_ISSET(c->fd, &sg->excepts))
		toret |= SG_FLAG_EXCEPT;
	return toret;
}
static void selector_group_copy(selector_group *dest,
				const selector_group *src) {
	memcpy(&dest->reads, &src->reads, sizeof(dest->reads));
	memcpy(&dest->sends, &src->sends, sizeof(dest->sends));
	memcpy(&dest->excepts, &src->excepts, sizeof(dest->excepts));
	dest->max = src->max;
}
static int selector_group_select(selector_group *sg,
				struct timeval *timeout)
{
	int res;
reselect:
	res = select(sg->max, &sg->reads, &sg->sends, &sg->excepts, timeout);
	if(res < 0) {
		switch(errno) {
		case EINTR:
			goto reselect;
		default:
			break;
		}
		/* Propogate an *error* */
		return -1;
	}
	return res;
}

/****************************/
/* swamp_selector functions */
/****************************/

/* Create/destroy a "selector" */
swamp_selector *swamp_selector_new(void)
{
	swamp_selector *ss = malloc(sizeof(swamp_selector));
	if(!ss)
		return NULL;
	selector_group_clear(&ss->done);
	selector_group_clear(&ss->next);
	return ss;
}
void swamp_selector_free(swamp_selector *ss)
{
	free(ss);
}

/* Perform a select operation */
int swamp_selector_select(swamp_selector *ss, int use_timeout,
				unsigned long timeout_msecs)
{
	struct timeval timeout;

	timeout.tv_sec = timeout_msecs / 1000;
	timeout.tv_usec = (timeout_msecs % 1000) * 1000;
	/* Move "next" to "done" */
	selector_group_copy(&ss->done, &ss->next);
	/* Clear "next" */
	selector_group_clear(&ss->next);
	/* Select on "done" */
	return selector_group_select(&ss->done,
			(use_timeout ? &timeout : NULL));
}

/* Add a connection to a selector */
int swamp_selector_add_connection(swamp_selector *ss,
				const swamp_connection *c)
{
	if(c->fd < 0)
		return 0;
	selector_group_add_conn(&ss->next, c);
	return 1;
}

/* Perform connection I/O based on a selector's results */
int swamp_selector_connection_io(swamp_selector *ss,
				swamp_connection *c)
{
	unsigned int code = selector_group_get_conn(&ss->done, c);
	if(code & SG_FLAG_EXCEPT)
		return 0;
	/* do reads if warranted */
	if((code & SG_FLAG_READ) && (swamp_fifo_from_fd(c->fifo_in,
					c->fd) <= 0))
		return 0;
	if((code & SG_FLAG_SEND) && (swamp_fifo_to_fd(c->fifo_out,
					c->fd) <= 0))
		return 0;
	return 1;
}
