/* License file snipped from here. Please see the "LICENSE" file, or the top of
 * the "swamp.h" header. Both contain the current license. */

#ifndef HEADER_SWAMP_CONF_H
#define HEADER_SWAMP_CONF_H

/* Learn about "dist_pattern" */
#include "dist_pattern.h"

/***************************/
/* Default limits/settings */
/***************************/

/* IMPORTANT NOTE: For now, "CHAR_STATUS" is no longer supported. The network
 * abstraction has removed a huge amount of network logic-sludge, which involved
 * removing all the code that the intricate little case-handling for this. It
 * may be reimplemented in the future. */
/* If CHAR_STATUS is defined, characters are printed out during progress;
 *   + = connect()
 *   - = close()
 *   X = connect error
 *   . = success
 *   * = dubious success
 *   # = generic error
 *   F = certificate verification failed
 */
/* #define CHAR_STATUS */

/* If DATE_OUTPUT is defined, the date/time the testing begins is output before
 * the tests start and likewise the date/time of the completion is output last.
 * Useful when many simultaneous instances are being run and the output is being
 * piped to files. */
#define DATE_OUTPUT

/* The maximum number of connections we will allocate and connect
 * with during the test. */
#define MAX_LIST_SIZE 100
/* The maximum finite limit we will put on requests (0 = let it
 * run indefinately). */
#define MAX_TOTAL_MAX 2000000
/* The maximum finite time limit we will put on the running time
 * (0 = let it run indefiniately). */
#define MAX_TIME_MAX 86400 /* one day */
/* The maximum allowed response size we will accept. */
#define MAX_RESPONSE_SIZE 65536
/* An "expectation" amount which effectively means we will keep accepting
 * data until the server closes the connection. */
#define EXPECT_SERVER_CLOSE ((unsigned int)-1)
/* The maximum number of seconds between stdout updates. */
#define MAX_PERIOD_UPDATE 300
/* The size we should make the FIFO arrays */
#define BUFFER_SIZE 2048

/********************************************/
/* "swamp_config" declaration and functions */
/********************************************/

typedef struct st_swamp_config {
	/* OpenSSL "SSL_CTX" settings */
	const char *cacert;
	const char *cert;
	const char *cipher_string;
#ifndef SWAMP_WITH_NO_ENGINE_SUPPORT
	const char *engine_id;
#endif
	/* The number of "swamp_item"s in a "swamp_thread_ctx" */
	unsigned long list_size;
	/* If non-zero, the maximum number of requests before stopping */
	unsigned long total_max;
	/* If non-zero, the maximum number of seconds before stopping */
	unsigned long time_max;
	/* The (minimum) size to allocate our response buffer */
	unsigned long response_size;
	/* How much response to expect before we are satisfied */
	unsigned long response_expected;
	/* What request string to send to servers */
	const char *request_string;
	/* The sequence of "s" and "r" items to control SSL_SESSION behaviour */
	const char *session_string;
	/* A cached value of the string's length */
	unsigned int session_string_length;
	/* A line of statistics is printed after this many seconds */
	unsigned long period_update;
	/* Whether the logo should be printed or not */
	unsigned int nologo;
	/* Whether each negotiated (and resumed) SSL_SESSION should be logged */
	unsigned int output_sessions;
	/* If non-NULL, comma-separated output is written each second */
	FILE *csv_output;
	/* The distribution pattern of 'servers' */
	dist_pattern *distribution;
} swamp_config;

/* Initialise/cleanup a 'swamp_config' structure */
void swamp_config_init(swamp_config *sc);
void swamp_config_finish(swamp_config *sc);

/* Process command-line input */
int swamp_config_process_command_line(swamp_config *sc,
			int argc, const char **argv);

/*******************************************/
/* command-line declarations and functions */
/*******************************************/

/* Macro to declare a command-line switch's name */
#define CMD_STR(s,v)	static const char CMD_STR_##s[] = v;
/* Macro to declare a matching enumerated item */
#define CMD_NUM(s)	CMD_NUM_##s
/* Macro to instantiate the avaiable command strings */
#define IMPLEMENT_CMDS_STRINGS_RAW \
	CMD_STR(SESSION_IDS, "-session_ids") \
	CMD_STR(NOLOGO, "-nologo") \
	CMD_STR(HELP1, "-h") \
	CMD_STR(HELP2, "-?") \
	CMD_STR(HELP3, "--help") \
	CMD_STR(CONNECT, "-connect") \
	CMD_STR(CAFILE, "-CAfile") \
	CMD_STR(CERT, "-cert") \
	CMD_STR(NUM, "-num") \
	CMD_STR(COUNT, "-count") \
	CMD_STR(TIME, "-time") \
	CMD_STR(EXPECT, "-expect") \
	CMD_STR(REQUEST, "-request") \
	CMD_STR(SESSION, "-session") \
	CMD_STR(UPDATE, "-update") \
	CMD_STR(CIPHER, "-cipher") \
	CMD_STR(CSV, "-csv") \
	CMD_STR(DISTRIBUTE, "-distribute")
#ifdef SWAMP_WITH_NO_ENGINE_SUPPORT
#define IMPLEMENT_CMDS_STRINGS IMPLEMENT_CMDS_STRINGS_RAW
#else
#define IMPLEMENT_CMDS_STRINGS \
	IMPLEMENT_CMDS_STRINGS_RAW \
	CMD_STR(ENGINE, "-engine")
#endif
/* Declare the corresponding command ids as an enumerated type */
typedef enum {
	CMD_NUM(SESSION_IDS),
	CMD_NUM(NOLOGO),
	CMD_NUM(HELP1),
	CMD_NUM(HELP2),
	CMD_NUM(HELP3),
	CMD_NUM(CONNECT),
	CMD_NUM(CAFILE),
	CMD_NUM(CERT),
	CMD_NUM(NUM),
	CMD_NUM(COUNT),
	CMD_NUM(TIME),
	CMD_NUM(EXPECT),
	CMD_NUM(REQUEST),
	CMD_NUM(SESSION),
	CMD_NUM(UPDATE),
	CMD_NUM(CIPHER),
	CMD_NUM(CSV),
	CMD_NUM(DISTRIBUTE),
#ifndef SWAMP_WITH_NO_ENGINE_SUPPORT
	CMD_NUM(ENGINE),
#endif
} cmd_id_t;
/* The structure each command-line switch is represented by */
typedef struct st_cmd_defn {
	const char *	cmd_name;
	cmd_id_t	cmd_id;
	unsigned int	cmd_args;
} cmd_defn;
/* Macros to declare a command-line switches with zero or one arguments */
#define CMD0(s)		{CMD_STR_##s, CMD_NUM_##s, 0}
#define CMD1(s)		{CMD_STR_##s, CMD_NUM_##s, 1}
/* Finally, a macro to instantiate the supported list of command-switches */
#define IMPLEMENT_CMDS_RAW \
	IMPLEMENT_CMDS_STRINGS \
	static const cmd_defn cmds[] = { \
	/* Commands that take no arguments */ \
	CMD0(SESSION_IDS), CMD0(NOLOGO), CMD0(HELP1), CMD0(HELP2), CMD0(HELP3), \
	/* Commands that take a single argument */ \
	CMD1(CONNECT), CMD1(CAFILE), CMD1(CERT), CMD1(NUM), CMD1(COUNT), \
	CMD1(TIME), CMD1(EXPECT), CMD1(REQUEST), CMD1(SESSION), CMD1(UPDATE), \
	CMD1(CIPHER), CMD1(CSV), CMD1(DISTRIBUTE),
#ifdef SWAMP_WITH_NO_ENGINE_SUPPORT
#define IMPLEMENT_CMDS \
	IMPLEMENT_CMDS_RAW \
	{NULL,0,0} }
#else
#define IMPLEMENT_CMDS \
	IMPLEMENT_CMDS_RAW \
	CMD1(ENGINE), \
	{NULL,0,0} }
#endif

#endif /* !defined(HEADER_SWAMP_CONF_H) */
